function [ FCrxn, FCindex, Rxnindex ] = Fullycoupledrxnsets( model )
% finds and isolates fully coupled reaction sets 
% FULLY COUPLED REACTION INDENTIFICATION
% 
%   INPUT
%   model (the following fields are required - others can be supplied)
%   rxns         Model reaction list
%   mets         Model metabolite list
%   S            Stoichiometric matrix
%   rev          Reversibility of reactions
%   
%   OUTPUT
%   FCrxn        A matrix where each row gives the set of reaction(IDs) which
%               are fully coupled together in a set
%   
%   FCindex     A matrix which replaces the reaction IDs of FCrxn with
%               index number of the reaction in the model.rxns
%
%   Rxnindex    A column matrix containing only the index numbers from the
%               model.rxns 
%   
%
%   Change nomenclature of files as per recquirement in FFCA program

    model.Reactions = model.rxns;
    model.Metabolites = model.mets;
    model.stoichiometricMatrix = model.S;
    model.reversibilityVector = model.rev;
    
%   running the FFCA program
%   Requires a linear programmer
%   fctable     output  of FFCA which gives a matrix of n x n dimension
%   where n is (the number of reactions  in  the model - blocked reactions). The entries correspond to 1 - fully coupled, 2 - 
    fctable = FFCA('glpk',model);
%   network    part of the network containing only working reactions, blocked reactins are removed
    [network] = correctReversibilities('glpk', model);
    clear model.Reactions model.Metabolites model.stoichiometricMatrix model.reversibilityVector;
%   Extract fully coupled reactions from the fctable (matrix entry ==1)
    set = 1;
    for i = 1 : length(fctable(:,1))
        num = 2;
        for j = 1 : length(fctable(1,:))
            if (fctable(i,j) == 1 && i ~= j);
                fully_couple{set,1} = network.Reactions(i,1);
                fully_couple{set,num} = network.Reactions(j,1);
                num = num + 1;
            end
        end 
        if(num==2)
            set = set - 1;
        end
        set = set + 1;
    end
    clear i j num set fctable network;

    FCrxn(1,:) = fully_couple(1,:);
    g = 2;
    for i = 2 : length(fully_couple(:,1))
        for k = 1 : length(FCrxn(:,1))
            for h = 1 : length(FCrxn(1,:))
                TF = strcmp(fully_couple{i,1},FCrxn{k,h});
                if(TF == 1)
                    break;
                elseif(isempty(FCrxn{k,h}))
                    break;
                end
            end
            if(TF == 1)
                break;
            end
        end
        if (TF ~= 1)
            FCrxn(g,:) = fully_couple(i,:);	
            g = g + 1;
        end
    end
    clear h g i j k TF fully_couple;
%    Creating Rxnindex
    Rxnindex = zeros(length(model.lb));
    for j = 1 : length(model.lb)
        Rxnindex(j) = j;
    end
    clear j;
    %creating FCindex
    FCindex = zeros(size(FCrxn));
    %fc_num = zeros(m,n);
    for j = 1:length(FCrxn(:,1))
         for k = 1:length(FCrxn(1,:))
            if(~isempty(FCrxn{j,k}))
                for i = 1:length(model.lb)
                   TF = strcmp(model.rxns(i),FCrxn{j,k});
                   if(TF == 1)
                         FCindex(j,k) = i;
                         break;
                   end 
                end
             end
         end
    end    
    clear i j k m n TF ans model;
end

