function [ opt ] = LHSampling( model, min, max, FCindex,FCrxn, optimal, Rxnindex, osenseStr )

%   [ opt ] = LHSampling( model, min, max, FCindex, FCrxn, optimal,Rxnindex )
% 
%   Uses Latin hybercude method to uniformly sample metabolic space
%   
%   INPUT
%   model (the following are required
%   S            Stoichiometric matrix
%   b            Right hand side = dx/dt
%   c            Objective coefficients
%   lb           Lower bounds
%   ub           Upper bounds
%   rev          Reversibility of the reactions
%
%   min, max     FVA min and max values after application of loop and bound constraint
%   FCindex     A matrix which replaces the reaction IDs of FCrxn with
%               index number of the reaction in the model.rxns
%
%    FCrxn        A matrix where each row gives the set of reaction(IDs) which
%               are fully coupled together in a set
%
%   optimal     the value of bojective function
%   
%   RxnIndex    A column matrix containing only the index numbers from the
%               model.rxns 
%
%   OUTPUT
%   opt         variable showinh the number of optimal flux vectors have been collected
%   sample files for optimal and sub-optimal are saved in separate files.


% find the fully coupled group which has objective function, this group has to be excluded from sampling
if exist('osenseStr', 'var')
            if isempty(osenseStr)
                osenseStr = 'max';
            end
        else
            osenseStr = 'max';
end  

used = [];
ind  = find(model.c);
for i = 1 : length(FCindex(:,1))
	  A = ismember(ind,FCindex(i,:));
	  [row, col] = find(A);	
	if(~isempty(row))
        for k = 1: length(FCindex(i,:))
            if(FCindex(i,k)~= 0)
                used(k,1) = FCindex(i,k);
            else
                break;
            end
        end
     break;
    end
end
clear ind i A col row k;
%sampling - calling recursive function

    opt = 1;
    sub_opt = 1;
    while(opt<10000)
        opt_each = 0;
         num = int32(round(rand(1)*length(model.lb)));
	 if(num == 0)
             num = length(model.lb);
          end
         if(~ismember(num,used))
                  if(min(num,1)~= max(num,1))
                  %not including the reactions which have constant flux through them which may be zero as well
                      [opt, sub_opt, opt_each] = LHsamplingcall(model, min, max, optimal, num, opt, used, FCindex, Rxnindex, sub_opt, opt_each, osenseStr);
                  end
             clear num opt_each; 
        end
    end
    clear model min max optimal num opt used FCindex Rxnindex sub_opt opt_each ans ;
end

