function [ opt, sub_opt, opt_each ] = LHsamplingcall( model, min, max, optimal, num, opt, used, FCindex, Rxnindex, sub_opt, opt_each , osenseStr)
%
%[ opt, sub_opt, opt_each ] = %LHsamplingcall( model, min, max, optimal, num, opt, used, FCindex, Rxnindex, sub_opt, opt_each )
%   
%   INPUT
%   model (the following fields are required - others can be supplied)
%   S            Stoichiometric matrix
%   b            Right hand side = dx/dt
%   c            Objective coefficients
%   lb           Lower bounds
%   ub           Upper bounds
%   rev          Reversibility
%   
%   min, max     FVA min and max values after application of loop and bound constraint
%
%   optimal     the value of objective function
%
%   num         index of the reactio choosen to be fixed
%
%   opt         number of optimals sampled
%
%   used        index numbers of reactions already fixed
%
%   FCindex     A matrix which replaces the reaction IDs of FCrxn with
%               index number of the reaction in the model.rxns
%
%   RxnIndex    A column matrix containing only the index numbers from the
%               model.rxns 
%   sub_opt     count of the number of suboptimal collected
%
%   opt_each    count of samples in each iteration; not more than 50 are  collected from one iteration altough normally less then 50 optimals are
%   found

%   adding the reaction to list of used reactions
%   Checking if the reaction 'num' is in any set of Fully coupled reactions
     A = ismember(FCindex, num);
    [row, col] = find(A);
    clear col;
    clear A;
%   Saving the position of reaction id in model.rxns to get the position of min and max for each reaction
    if(~isempty(row))
        A = ismember(Rxnindex,FCindex(row,:));
        num_id = find(A);
       for i = 1 : length(num_id)
           used(length(used)+1) = num_id(i);
       end
    else
        used(length(used)+1) = num;
    end
   
    clear A num_id row i;
%   lhs_sampling start - vector X with 10 values between 0 and 1 at regular intervals
    X = lhsdesign(10,1);
        %X
        for h = 1:10
            
            %set flux value between the range of min_new and max_new
            flux = (min(num,1) + (max(num,1)-min(num,1))*X(h));
            Model_changed = changeRxnBounds(model, model.rxns(num,1),flux);
            %optimize with changed model
            fba_model_changed = optimizeCbModel(Model_changed, osenseStr);
            clear flux;
            % only id model is feasible enter if
            if(~isempty(fba_model_changed.x))
                model_opt = (round((fba_model_changed.f)*1000000))/1000000;
                %save sub-optimal
                if(model_opt < optimal)
                    if(sub_opt <= 10000)
                        val = 1;
                        for u = 1:length(model.lb)
                            n = model.rev(u);
                            if(n == 0)
                                if(fba_model_changed.x(u) < 0)
                                    val = 2;
                                    break;
                                end
                            end
                            clear n;
                        end
                        if(val == 1)
                             sample_all(1,:) = fba_model_changed.x;
                             sample_all = [sample_all, model_opt];
                             dlmwrite('Sampling_suboptimal.m',sample_all ,'-append');
                             sub_opt = sub_opt + 1;
                             clear sample_all;
                        end
                        clear u val fba_model_changed;
                    end
                end
                if(model_opt == optimal)
                    if(opt <= 10000) 
                        val = 1;
                        for u = 1:length(model.lb)
                            n = model.rev(u);
                            if(n == 0)
                                if(fba_model_changed.x(u) < 0)
                                    val = 2;
                                    break;
                                end
                            end
                            clear n;
                        end
                        if(val == 1)
                            sample_opt(1,:) = fba_model_changed.x;
                            sample_opt = [sample_opt, model_opt];
                             dlmwrite('Sampling_optimal.m',sample_opt ,'-append');
                             clear sample_opt n u fba_model_changed model_opt;
                             opt = opt + 1
                             pause(1)
                             opt_each = opt_each + 1;
                             used_loop = used;
                             clear u val fba_model_changed;
     %      Call function for next iteration
     %      not more than 50 samples from an iteration
                             if(opt_each<50)
                                 if(length(used) == length(model.lb))
                                    break;
                                else
                                    no = int32(round(rand(1)*length(model.lb)));
                                    if(no == 0)
                                        no = length(model.lb);
                                    end
                                    while((ismember(no,used_loop)) || (min(no,1)== max(no,1)))
                                        no =int32(round(rand(1)*length(model.lb)));
                                        if(no == 0)
                                            no = length(model.lb);
                                        end				    
                                    end
                                    %disp 'next cycle';
                                    [opt, sub_opt, opt_each] = LHsamplingcall(Model_changed, min, max, optimal, no, opt, used_loop, FCindex, Rxnindex, sub_opt, opt_each, osenseStr);
                                    clear no used_loop Model_changed;
                                    %disp 'exit';
                                 end
                             end
                        else
                            break;
                        end
                    end
                end
            else
                % disp 'model infeasible';
            end
            clear Model_changed;
            clear fba_model_changed;
            clear model_opt;
        end
        clear Model_new min ans max optimal no num used FCindex Rxnindex model X h osenseStr;
end


