function [Model_loop_removed, min_original, max_original optimal] = Preprocessing_loop_removal (model, osenseStr) 
    %   preprocessing includes two steps
    %    i)finding and removing loops
    %   ii) setting upper and lower bound
    %   
    % FINDING AND REMOVING LOOP REACTIONS
    %   [Model_loop_removed, min_original, max_original optimal] = LHSpreprocessing (model, osenseStr) 
    %
    %   INPUT
    %   model(the following fields are required)
    %   S     Stoichiometric matrix
    %   b     Right hand side = dx/dt
    %   c     Objective coefficiets
    %   lb    Lower bounds
    %   ub    Upper bounds
    %
    %   OPTIONAL INPUT
    %   osenseStr      Maximize ('max')/minimize ('min') (opt, default = 'max')
    %
    %   OUTPUT
    %   Model_loop_removed   The model after removal of loops
    %   min_original     FVA minimum for initial model
    %   max_original     FVA maximum for initial model
    %   optimal     Value of the objective function

    %   check for optioanl input/ set default value
        if exist('osenseStr', 'var')
            if isempty(osenseStr)
                osenseStr = 'max';
            end
        else
            osenseStr = 'max';
        end
    %   calculating optimum objective functiom
        fba_model = optimizeCbModel(model,osenseStr);
        optimal = (round((fba_model.f)*1000000))/1000000;
        clear fba_model;

    %   flux variability analysis
        [min_original,max_original]=fluxVariability(model, 0);

    %   remove loop from the model
            % loop_rxn_no - gives the number of reactions involved in looping (same use as try4)
            % loop_rxn_id - array containing id of reaction involved in loopes
            % num - counting variable ( can be used again)
            % min_loop, max_loop    FVA results for intermediate models as loops ae removed one by one
            % Model_loope_removed  - name Of model after loope reactions have been removed(final model to be used in next program)
            % Model_changed - just a variable to save the model intermediate
            % fba_at_0 - optimized flux at flux 0
            % flux_opt_0 - optimal solution at flux through reaction(id) 0.1
            % fba_at_01_less - optimized flux at flux- 0.1
            % flux_opt_01_less - optimal solution at flux through reaction(id) -0.1
            % fba_at_01_more - optimized flux at flux 0.1
            % flux_opt_01_more - optimal solution at flux through reaction(id) 0.1
            % model_opt - optimal solution when the flux for the loope
            % reaction has been changes
        Model_loop_removed = model;
        loop_rxn_no = 0;
        while(loop_rxn_no ~=1)        
            clear loop_rxn_no;
            [min_loop,max_loop] = fluxVariability(Model_loop_removed, 0);
            loop_rxn_no = 0;
            num=1;
            while(num<=length(model.lb))
                %so that reactions with constant flux are not included(lb=ub=min=max)
                if(min_loop(num,1)~= max_loop(num,1))
                    if((min_loop(num,1) <= (min(model.lb)+1) || max_loop(num,1) >= (max(model.ub)-1)))
                        loop_rxn_no = loop_rxn_no + 1;
                        loop_rxn_id(loop_rxn_no,1) = model.rxns(num,1);
                    end
                end
                num = num + 1;	
            end
            clear min_loop max_loop num ans;
            %change the flux to the minimum value such that it still gives the optimal solution
            if(loop_rxn_no~=0)
                    num = 1;
                    flux = 0;
                    %model change and calculation of optimal at flux through loope reactions to be zero and optimizationloope_rxn_no
                    Model_loop_removed  = changeRxnBounds (Model_loop_removed ,loop_rxn_id(num,1),0);
                    fba_at_0= optimizeCbModel(Model_loop_removed,osenseStr );
                    flux_opt_0 = (round((fba_at_0.f)*1000000))/1000000;
                    model_opt = flux_opt_0;
                    clear fba_at_0;
                        if(flux_opt_0 ~= optimal)
                            %model change and calculation of optimal at flux through loope reactions to be 0.1 and optimization
                            Model_changed  = changeRxnBounds (Model_loop_removed ,loop_rxn_id(num,1),0.1);
                            fba_at_01_more= optimizeCbModel(Model_changed,osenseStr);
                            flux_opt_01_more = (round((fba_at_01_more.f)*1000000))/1000000;
                            clear Model_changed fba_at_01_more ans;
                            %model change and calculation of optimal at flux through loope reactions to be -0.1 and optimization
                            Model_changed  = changeRxnBounds (Model_loop_removed ,loop_rxn_id(num,1),-0.1);
                            fba_at_01_less= optimizeCbModel(Model_changed,osenseStr);
                            flux_opt_01_less = (round((fba_at_01_less.f)*1000000))/1000000;
                            clear Model_changed fba_at_01_less ans;
                            %change flux for the loope reactions till the obtain a value which is giving optimal flux
                            if(flux_opt_0 < optimal)
                                if( flux_opt_01_more > flux_opt_0 && flux_opt_01_less < flux_opt_0)
                                    while(model_opt~=optimal||model_opt>optimal)
                                        flux = flux + 0.0001;
                                        Model_loop_removed = changeRxnBounds (Model_loop_removed,loop_rxn_id(num,1),flux);
                                        fba_new = optimizeCbModel(Model_loop_removed,osenseStr);
                                        model_opt = (round((fba_new.f)*1000000))/1000000;
                                        clear fba_new ans;
                                    end
                               elseif ( flux_opt_01_more < flux_opt_0 && flux_opt_01_less > flux_opt_0)
                                    while(model_opt~=optimal||model_opt>optimal)
                                        flux = flux - 0.0001;
                                        Model_loop_removed  = changeRxnBounds (Model_loop_removed,loop_rxn_id(num,1),flux);
                                        fba_new = optimizeCbModel(Model_loop_removed,osenseStr );
                                        fba_new.f;
                                        model_opt = (round((fba_new.f)*1000000))/1000000;
                                        clear fba_new ans;
                                    end
                                end
                            else
                                if( flux_opt_01_more > flux_opt_0 && flux_opt_01_less < flux_opt_0)
                                    while(model_opt~=optimal||model_opt<optimal)
                                        flux = flux - 0.0001;
                                        Model_loop_removed  = changeRxnBounds (Model_loop_removed ,loop_rxn_id(num,1),flux);
                                        fba_new = optimizeCbModel(Model_loop_removed,osenseStr );
                                        model_opt = (round((fba_new.f)*1000000))/1000000;
                                        clear fba_new ans;
                                    end
                                 elseif ( flux_opt_01_more < flux_opt_0 && flux_opt_01_less > flux_opt_0)
                                    while(model_opt~=optimal||model_opt<optimal)
                                        flux = flux + 0.0001;
                                        Model_loop_removed  = changeRxnBounds (Model_loop_removed,loop_rxn_id(num,1),flux);
                                        fba_new = optimizeCbModel(Model_loop_removed );
                                        model_opt = (round((fba_new.f)*1000000))/1000000;
                                        clear fba_new ans;
                                     end
                                end
                            end
                           clear  flux_opt_01_more flux_opt_01_less model_opt ans;
                        end
            else
                loop_rxn_no = loop_rxn_no + 1;        
            end
            clear flux flux_opt_0 loop_rxn_id num num_id ans;
        end
         clear loop_rxn_no osenseStr model ans;
end