function [Model_new, min_new, max_new] = Preprocessing_setbounds( model, osenseStr )
 %   preprocessing includes two steps
    %    i)finding and removing loops
    %   ii) setting upper and lower bound
    %   
    % SETTING UPPER AND LOWER BOUNDS
    %   [min_new, max_new,Model_new] = LHSpreprocessing ( model)
    %
    %   INPUT
    %   model(the following fields are required)- after loop removal
    %   S     Stoichiometric matrix
    %   b     Right hand side = dx/dt
    %   c     Objective coefficiets
    %   lb    Lower bounds
    %   ub    Upper bounds
    %
    %   OPTIONAL INPUT
    %   osenseStr      Maximize ('max')/minimize ('min') (opt, default = 'max')
    %
    %   OUTPUT
    %   Model_new   The final model after application of both steps
    %   min_new     final FVA minimum flux after application of both steps
    %   max_new     final FVA maximum flux after application of both steps
    %   min_orignal     FVA minimum for initial model
    %   max_orignal     FVA maximum for initial model
    %   optimal     Value of the objective function
    %   BOUND CONSTRAINT
    %   CHANGE THE UPPER BOUND OF ALL THE REACTIONS TO A VALUE OF MAX + 0.1 TO REDUCE THE SEARCH SPACE (EXCEPT THOSE REACTIONS FOR WHICH FLUX IS FIXED OR IN ALREADY MAXIMUM)
        %change_ub_no = number of reactions  for which the ub is changes to max + 1 
        %change_ub_id = the cell array of ids for which the reaction ub is being changed to max + 1
        %change_ub_value = stores the values for the fluxes corresponding to the reactions in change_ub_id
        
        %   check for optioanl input/ set default value
        if exist('osenseStr', 'var')
            if isempty(osenseStr)
                osenseStr = 'max';
            end
        else
            osenseStr = 'max';
        end
        
        [min_loop_removed, max_loop_removed] = fluxVariability(model, 0);
        
        num = 1;
        change_ub_no = 0;
        change_lb_no = 0;
        while(num<=length(model.lb))
            if(max_loop_removed(num)~= model.ub(num,1))
                change_ub_no = change_ub_no + 1;
                change_ub_id(change_ub_no,1) = model.rxns(num,1);
                change_ub_value(change_ub_no,1) = max_loop_removed(num) + 0.1;
            end
            if(min_loop_removed(num)~= model.lb(num,1))
		 n = model.rev(num);
                if(n == 0 && min_loop_removed(num) == 0)
			;
                else
			change_lb_no = change_lb_no + 1;
			change_lb_id(change_lb_no,1) = model.rxns(num,1);
			change_lb_value(change_lb_no,1) = min_loop_removed(num) - 0.1;
		end
	    end
            num = num + 1;
        end
    
        Model_change_up = changeRxnBounds(model, change_ub_id,change_ub_value,'u');

        fba_model_ub = optimizeCbModel(Model_change_up, osenseStr);
        optimal_model_new = (round((fba_model_ub.f)*1000000))/1000000;
        clear fba_model_ub;

        if(optimal_model_new == optimal)
            Model_new = changeRxnBounds(Model_change_up, change_lb_id,change_lb_value,'l');
            clear change_ub_no change_ub_id change_ub_value change_lb_no change_lb_id change_lb_value num Model_change_up;
            %flux balance analysis
            fba_Model_new = optimizeCbModel(Model_new, osenseStr);
            optimal_model_new = (round((fba_Model_new.f)*1000000))/1000000;
        else
            disp 'Upper bound constraints needs to be relaxed';
        end

        if(optimal_model_new == optimal)
            %flux variability of the new model
            [min_new,max_new] = fluxVariability(Model_new, 0);
            clear optimal_model_new;
        else
            disp 'Lower bound constraints needs to be relaxed';
        end
        clear model max_loop_removed min_loop_removed osenseStr model fba_Model_new;
end

